#!/usr/bin/perl
# First attempt at a perl script looks up all active leases.
use LibNAC;
use Net::ISC::DHCPd::OMAPI;
package LibNAC::SessionList;

sub nac_session_list {

	## Initial Declarations
	open (STDERR,">/dev/null");
	my ($dhcp_primary) = "169.229.7.8";
	my ($dhcp_secondary) = "169.229.7.9";

	my @active_ip;
	my @states;

	my $omapi_port = 20003;
	my $omapi_key = 'OMAPIMgmt 49KvvSHeNHKR0yy97g2+5qZCxc0lwDEyEEaVf1x/bl8='; 

	## Server Connections
	my $omapi_primary = Net::ISC::DHCPd::OMAPI->new(server => "$dhcp_primary", 
		port => "$omapi_port",
		key => "$omapi_key") || die "failed to intialize primary omapi server\n";

	$connection_status_primary = $omapi_primary->connect();

	$omapi_active = $omapi_primary;

	my $omapi_secondary = Net::ISC::DHCPd::OMAPI->new(server => "$dhcp_secondary",
		port => "$omapi_port",
		key => "$omapi_key" ) || "failed to initialize secondary omapi server\n";
	$connection_status_secondary = $omapi_secondary->connect() or die "Failed to connect to the secondary\n";

	## Error Handling (if one server is down)
	if ($omapi_status_primary == 1 && $omapi_status_secondary == 1) {
		print "No usable omapi connections, exiting.\n";
		exit 1;
	} elsif ($omapi_status_primary == 1) {
		$omapi_primary = $omapi_secondary;
	} elsif ($omapi_status_secondary == 1) {
		$omapi_secondary = $omapi_primary;
	}
	

	# First, we need a list of all sessions
	my %config = LibNAC::nac_readconf();

	my @nac_public_nets =split(/[\s]+/,$config{"NAC_PUBLIC_NETS"});

	foreach my $pub_net (@nac_public_nets) {
		unless ($pub_net=~ m/^[\s]*$/){
			my $client_range = LibNAC::nac_client_ranges($pub_net);
			my ($lower_bound,$upper_bound) = split(/\-/,$client_range);
			my $count = LibNAC::nac_ip_to_num($upper_bound) - 
				LibNAC::nac_ip_to_num($lower_bound);
			my ($ip) = $lower_bound;
			for (my $i = 0; $i <= $count; $i=$i+2 ) {
				my ($lease) = $omapi_primary->new_object("lease");
				$lease->ip_address($ip);
				$lease->read;
				my ($state) = $lease->state;
				push (@states,$state);
				if ($state eq "active") {
					push (@active_ip,"$ip");
				}
				$ip = increment_ip($ip);
				my ($lease) = $omapi_secondary->new_object(lease =>(ip_address=>$ip));
				$lease->read;
				my ($state) = $lease->state;
				push (@states,$state);
				if ($state eq "active") {
					push (@active_ip,"$ip");
				}
				$ip = increment_ip($ip);
			}
		}
	}

	return \@active_ip;
}


sub increment_ip {
	my $ip = shift;
	my ($oct1,$oct2,$oct3,$oct4) = split(/\./,$ip);
	$oct4++;
	if ($oct4 > 255) {
		$oct4 = 0;
		$oct3++;
	}
	if ($oct3 > 255) {
		$oct3 = 0;
		$oct2++;
	}
	return "$oct1.$oct2.$oct3.$oct4";
}
